import sys
import os
import argparse
import pyAMI


def main():
    parser = argparse.ArgumentParser(usage='%(prog)s [options] <App.apk | AndroidManifest.xml>')
    parser.add_argument("-x", "--xml", action="store_true", default=False)
    parser.add_argument("--perms", action="store_true", default=False)
    parser.add_argument("--exports", action="store_true", default=False)
    parser.add_argument("target",
            help='Apk or AndroidManifest.xml file')

    args     = parser.parse_args()
    target   = args.target
    manifest = pyAMI.Manifest.open(target)

    if not manifest:
        print("Unable to open '{}'".format(target))
        sys.exit(1)

    print("{}: {} - {}".format(manifest.package_name, manifest.version_name, manifest.version_code))
    if manifest.application:
        if manifest.application.cls:
            print("Application: {}".format(manifest.application.cls))
    print("")

    if args.perms:
        for p in manifest.permissions:
            print(p)

    if args.xml:
        print(manifest.xml)

    if args.exports:
        if len(manifest.services) > 0:
            print("== Services ==")
            for service in manifest.services:
                print(f"    {service.cls}")
            print("")

        if len(manifest.receivers) > 0:
            print("== Receivers ==")
            for br in manifest.receivers:
                title = f"    {br.cls}"
                if br.has_permission:
                    title += f" [{br.permission}]"
                if br.is_exported:
                    title += f" EXPORTED"

                if not br.is_enabled:
                    title += f" DISABLED"
                print(title)
                if len(br.filters) > 0:
                    actions = []
                    data = []
                    categories = []
                    for f in br.filters:
                        actions += f.actions
                        categories += f.categories
                        data += [str(d) for d in f.data]

                    if actions:
                        print(" " * 4, "-> Actions")
                        for action in actions:
                            print(" " * 8, f"- {action}")

                    if categories:
                        print(" " * 4, "-> Categories")
                        for cat in categories:
                            print(" " * 8, f"- {cat}")

                    if data:
                        print(" " * 4, "-> Data")
                        for data in data:
                            print(" " * 8, f"- {data!s}")
            print("")


        if len(manifest.providers) > 0:
            print("== Providers ==")
            for provider in manifest.providers:
                print(f"    {provider.cls:<70}: content://{provider.authorities}/")
            print("")



if __name__ == "__main__":
    main()

