/* Copyright 2018 R. Thomas
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#include "AMI/AMI.hpp"
#include "AndroidManifestImpl.hpp"
#include "macros.h"

#include <algorithm>

namespace AMI {

AXML::~AXML(void) = default;

AXML AXML::open(const std::string& path) {
  std::unique_ptr<details::AndroidManifestImpl> impl{details::AndroidManifestImpl::open(path)};
  return std::move(impl);
}

AXML AXML::open(const char* raw, size_t size) {
  return AXML::blob_t{raw, raw + size};
}

AXML AXML::open(const AXML::blob_t& blob) {
  return blob;
}

AXML AXML::open(AXML::blob_t&& blob) {
  return blob;
}

AXML::AXML(const AXML::blob_t& blob) :
  pimpl_{new details::AndroidManifestImpl{blob.data(), blob.size()}}
{
}

AXML::AXML(const AXML& other) :
  pimpl_{nullptr}
{
  if (other) {
    this->pimpl_ = std::unique_ptr<details::AndroidManifestImpl>{
      new details::AndroidManifestImpl{*other.pimpl_}
    };
  }
}

XML::Root& AXML::xml(void) {
  return this->pimpl_->root();
}

const XML::Root& AXML::xml(void) const {
  return this->pimpl_->root();
}

AXML& AXML::operator=(const AXML& other) {
  if (*this and this != &other) {
    this->pimpl_.reset(new details::AndroidManifestImpl{*other.pimpl_});
  }
  return *this;
}

AXML::AXML(std::unique_ptr<details::AndroidManifestImpl>&& impl) :
  pimpl_{std::move(impl)}
{}

AXML::operator bool() const {
  return this->pimpl_ != nullptr and static_cast<bool>(*this->pimpl_);
}

std::ostream& operator<<(std::ostream& os, const AXML& m) {
  return os;
}


size_t AXML::string_pool_size(void) const {
  return this->pimpl_->string_pool_size();
}


const AXML::string_pool_t& AXML::string_pool(void) const {
  return this->pimpl_->string_pool();
}

} // Namespace AMI
