/* Copyright 2018 R. Thomas
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "AMI/AMI.hpp"
#include "AndroidManifestImpl.hpp"
#include "macros.h"

namespace AMI {

Provider::Provider(const XML::Element& e) :
  element_{const_cast<XML::Element*>(&e)}
{}

Provider::Provider(const Provider&) = default;
Provider& Provider::operator=(const Provider&) = default;

bool Provider::is_valid(void) const {
  return this->element_ != nullptr;
}

Provider::operator bool() const {
  return this->is_valid();
}

std::string Provider::cls(void) const {
  if (not this->is_valid()) {
    return "";
  }
  const XML::Attribute* attr = (*this->element_)["android:name"];
  if (attr == nullptr) {
    return "";
  }
  return attr->svalue();
}

bool Provider::is_exported(void) const {
  if (not this->is_valid()) {
    return true;
  }

  const XML::Attribute* attr = (*this->element_)["android:exported"];
  if (attr == nullptr) {
    return true;
  }
  return static_cast<bool>(attr->ivalue());
}

bool Provider::is_enabled(void) const {
  if (not this->is_valid()) {
    return true;
  }

  const XML::Attribute* attr = (*this->element_)["android:enabled"];
  if (attr == nullptr) { /* Default value */
    return true;
  }
  return static_cast<bool>(attr->ivalue());
}

bool Provider::is_direct_boot_aware(void) const {
  if (not this->is_valid()) {
    return false;
  }

  const XML::Attribute* attr = (*this->element_)["android:directBootAware"];
  if (attr == nullptr) { /* Default value */
    return false;
  }
  return static_cast<bool>(attr->ivalue());
}

std::string Provider::authorities(void) const {
  if (not this->is_valid()) {
    return "";
  }

  const XML::Attribute* attr = (*this->element_)["android:authorities"];
  if (attr == nullptr) { /* Default value */
    return "";
  }
  return attr->svalue();
}



std::ostream& operator<<(std::ostream& os, const Provider& m) {
  //static const char* SPACE = "   ";
  //os << m.cls();
  //if (m.is_exported()) {
  //  os << " - EXPORTED";
  //}

  //auto&& actions = m.actions();
  //if (actions.size() > 0) {
  //  os << std::endl;
  //  for (const std::string& action : m.actions()) {
  //    os << SPACE << action << std::endl;
  //  }
  //}
  return os;
}


}
